const webpack = require('webpack');
const path = require('path');
const HtmlWebpackPlugin = require('html-webpack-plugin');
const HtmlWebpackHarddiskPlugin = require('html-webpack-harddisk-plugin');
const fs = require('fs');
const util = require('util');

const rawConfig = {};
['react'].forEach((framework) => {
    rawConfig[framework] = {};
    ['development', 'production'].forEach((env) => {
        const configFilePath = `./${framework}/webpack.config.${env}.js`;
        rawConfig[framework][env] = {
            processedTimes: -1,
            config: require(configFilePath),
        };
    });
});

module.exports = {
    react: (params) => {
        let env = params || {};
        const {
            entryPath,
            entries,
            vendors,
            publicPath,
            html,
            outputPath,
            configOnly,
        } = params || {};

        const {
            htmlTitle,
            faviconPath,
            htmlTemplatePath,
            htmlFileName,
            htmlOutputPath,
        } = html || {};

        if (!env) {
            env = (process.env.NODE_ENV || 'development').toLowerCase();
        } else if (typeof env === 'string') {
            env = env.toLowerCase();
        }
        if (['development', 'production'].indexOf(env) < 0) env = 'development';

        const pluginsAppended = 2;


        const configInMemory = rawConfig.react[env];
        configInMemory.processedTimes++;
        const config = configInMemory.config;

        // Replace configuration items according to the input parameters
        // Entry path of the App
        if (typeof config.entry.app === 'string') {
            config.entry.app = entryPath;
        } else if (Array.isArray(config.entry.app)) {
            config.entry.app[1] = entryPath;
        }

        // Vendors
        if (vendors && Array.isArray(vendors)) {
            if (configInMemory.processedTimes > 0 || !config.entry.vendors || !Array.isArray(config.entry.vendors)) {
                config.entry.vendors = [];
            }
            Array.prototype.push.apply(config.entry.vendors, vendors);
        }

        // Public path
        config.output.publicPath = publicPath;

        // Output
        config.output.path = outputPath;

        // HTML
        if (typeof htmlOutputPath === 'string'
            && typeof htmlTemplatePath === 'string'
        ) {
            const htmlConfig = {
                template: htmlTemplatePath,
                filename: htmlFileName,
                inject: false,
                alwaysWriteToDisk: true,
            };

            if (typeof htmlTitle === 'string') {
                htmlConfig.title = htmlTitle;
            }

            if (typeof faviconPath === 'string') {
                htmlConfig.favicon = faviconPath;
            }

            // remove plugins previously pushed
            if (configInMemory.processedTimes > 0) {
                config.plugins.splice(config.plugins.length - pluginsAppended);
            }
            config.plugins.push(new HtmlWebpackPlugin(htmlConfig));

            const hardDiskPluginOptions = {};
            if (htmlOutputPath) {
                hardDiskPluginOptions.outputPath = htmlOutputPath;
            }
            config.plugins.push(new HtmlWebpackHarddiskPlugin(hardDiskPluginOptions));
        }

        //
        const koaMiddlewares = [];

        if (!configOnly) {
            const webpack = require('webpack');
            const {devMiddleware, hotMiddleware} = require('koa-webpack-middleware');
            const compiler = webpack(config);

            const webpackDev = devMiddleware(compiler, {
                noInfo: false,
                publicPath: config.output.publicPath,
            });
            const webpackHot = hotMiddleware(compiler, {
                log: console.log,
                path: '/__webpack_hmr',
                heartbeat: 1 * 1000,
            });
            koaMiddlewares.push(webpackDev);
            koaMiddlewares.push(webpackHot);
        }

        return {
            config,
            koaMiddlewares,
        }
    }
};
